﻿using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Input;
using ThinkGeo.Core;

namespace MouseWheelOnMarker
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        SimpleMarkerOverlay simpleMarkerOverlay;
        public MainWindow()
        {
            InitializeComponent();
        }

        private void wpfMap_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap.MapUnit = GeographyUnit.Meter;

            ThinkGeoCloudVectorMapsOverlay thinkGeoCloudVectorMapsOverlay = new ThinkGeoCloudVectorMapsOverlay("itZGOI8oafZwmtxP-XGiMvfWJPPc-dX35DmESmLlQIU~", "bcaCzPpmOG6le2pUz5EAaEKYI-KSMny_WxEAe7gMNQgGeN9sqL12OA~~", ThinkGeoCloudVectorMapsMapType.Light);

            wpfMap.Overlays.Add(thinkGeoCloudVectorMapsOverlay);

            simpleMarkerOverlay = new SimpleMarkerOverlay();
            Marker marker = new Marker(new PointShape(-10776836.12, 3912348.04)) { Tag = 0 };
            marker.MouseWheel += Marker_MouseWheel;
            simpleMarkerOverlay.Markers.Add(marker);
            wpfMap.Overlays.Add(simpleMarkerOverlay);

            wpfMap.Refresh();
            wpfMap.CurrentScaleChanged += WpfMap_CurrentScaleChanged;
        }

        private void WpfMap_CurrentScaleChanged(object sender, CurrentScaleChangedMapViewEventArgs e)
        {
            simpleMarkerOverlay.Markers[0].Tag = 0;
        }

        private void Marker_MouseWheel(object sender, MouseWheelEventArgs e)
        {
            Marker marker = sender as Marker;
            int multiplier = (int)marker.Tag + 1;
            marker.Tag = multiplier;
            int currentIndex = wpfMap.GetSnappedZoomLevelIndex(wpfMap.CurrentScale);
            double targetScale = getTargetScale(e.Delta, multiplier, currentIndex);

            PointShape pointShape = new PointShape(marker.Position.X, marker.Position.Y);
            float mapWidth = (float)wpfMap.ActualWidth;
            float mapHeight = (float)wpfMap.ActualHeight;

            RectangleShape targetExtent = MapUtil.ZoomToScale(targetScale, wpfMap.CurrentExtent, GeographyUnit.Meter, mapWidth, mapHeight, MapUtil.ToScreenCoordinate(wpfMap.CurrentExtent, pointShape, mapWidth, mapHeight));

            wpfMap.ZoomTo(targetExtent.GetCenterPoint(), targetScale);
        }

        private double getTargetScale(int delta, int multiplier, int currentIndex)
        {
            Collection<ZoomLevel> zoomLevels = wpfMap.ZoomLevelSet.GetZoomLevels();

            int targetZoomLevelIndex = delta > 0 ? currentIndex + multiplier : currentIndex - multiplier;
            if (targetZoomLevelIndex < 0)
            {
                targetZoomLevelIndex = 0;
            }
            if (targetZoomLevelIndex >= zoomLevels.Count)
            {
                targetZoomLevelIndex = zoomLevels.Count - 1;
            }

            double targetScale = zoomLevels[targetZoomLevelIndex].Scale;
            return targetScale;
        }
    }
}
